import contextlib
from fastapi import FastAPI
from fastapi.responses import RedirectResponse
from mcp.server.fastmcp import FastMCP
from mcp.server.auth.settings import AuthSettings
from pydantic import AnyHttpUrl
from pydantic_settings import BaseSettings, SettingsConfigDict
from server.token_verifier.token_verifier import (
    IntrospectionTokenVerifier,
)


class ServerSettings(BaseSettings):
    """Settings for the MCP Server."""

    model_config = SettingsConfigDict(env_prefix="MCP_RESOURCE_")

    # Server settings
    host: str = "localhost"
    port: int = 8000
    server_url: AnyHttpUrl = AnyHttpUrl("http://localhost:8000")

    # Authorization Server settings
    auth_server_url: AnyHttpUrl = AnyHttpUrl("http://localhost:9000")
    auth_server_introspection_endpoint: str = "http://localhost:9000/introspect"

    # MCP settings
    mcp_scope: str = "user"

    # RFC 8707 resource validation
    oauth_strict: bool = False


def create_mcp_server(settings: ServerSettings = ServerSettings()) -> FastMCP:
    token_verifier = IntrospectionTokenVerifier(
        introspection_endpoint=settings.auth_server_introspection_endpoint,
        server_url=str(settings.server_url),
        validate_resource=settings.oauth_strict,  # Only validate when --oauth-strict is set
    )

    mcp: FastMCP = FastMCP(
        name="example-server",
        instructions="This server specializes in private operations of user profiles data",
        debug=True,
        # Auth configuration for RS mode
        token_verifier=token_verifier,
        auth=AuthSettings(
            issuer_url=settings.auth_server_url,
            required_scopes=[settings.mcp_scope],
            resource_server_url=settings.server_url,
        ),
    )

    @mcp.tool(
        name="set_user_profile",
        description="Set the user profile information in database from user data",
    )
    async def set_user_profile(data: dict) -> dict:
        """Set the user profile information in database for user_id"""

        # TODO: Implement any logic here. Eg. database call or request profile
        # information from your user auth system.

        return {
            "status": "success",
            "message": "User added to dataset successfully",
            "data": data,
        }

    return mcp


def create_app(
    servers: list[FastMCP],
    settings: ServerSettings = ServerSettings(),
) -> FastAPI:

    # Create a combined lifespan to manage both session managers
    @contextlib.asynccontextmanager
    async def lifespan(app: FastAPI):
        async with contextlib.AsyncExitStack() as stack:
            for server in servers:
                await stack.enter_async_context(server.session_manager.run())
            yield

    app = FastAPI(lifespan=lifespan)
    for server in servers:
        app.mount(
            f"/{server.name}",
            server.streamable_http_app(),
        )

    @app.get("/")
    async def redirect_to_help():
        return RedirectResponse(url="/aviable-servers")

    @app.get("/aviable-servers", include_in_schema=False)
    async def aviable_servers():
        return {
            "aviable-servers": [
                f"http://{settings.host}:{settings.port}/{server.name}/mcp"
                for server in servers
            ]
        }

    return app


server_settings: ServerSettings = ServerSettings()
app: FastAPI = create_app(
    servers=[create_mcp_server(settings=server_settings)],
    settings=server_settings,
)



from server.oauth_server import OAuthServer
from uvicorn import Config, Server
from .api import app, server_settings
import asyncio
import logging

logger = logging.getLogger(__name__)


def run_oauth_server():
    oauth_server: OAuthServer = OAuthServer()
    oauth_server.run_starlette_server()


def sample_fastapi_mcp_server():
    async def uvicorn_run():
        config = Config(
            app,
            host=server_settings.host,
            port=server_settings.port,
            log_level="info",
        )
        server = Server(config)
        logger.info(
            f"🚀 MCP Fastapi Server running on http://{server_settings.host}:{server_settings.port}"
        )
        await server.serve()

    asyncio.run(uvicorn_run())
    return app



from mcp.client.session import ClientSession
from mcp.shared.exceptions import McpError
from client.oauth_client import OAuthClient

import asyncio
from datetime import timedelta
from mcp.client.streamable_http import streamablehttp_client


def sample_mcp_client():
    # Es necesario tener un servidor MCP corriendo en la direccion http://127.0.0.1:8000//example-server/mcp por httpstream
    server_url: str = "http://127.0.0.1:8000/example-server/mcp"
    oauth_server_url: str = "http://127.0.0.1:9000"

    # Es necesario tener un servidor OAuth corriendo en la direccion http://localhost:9000
    oauth_client: OAuthClient = OAuthClient(
        client_name="sample_client",
        server_url=oauth_server_url,
        authorized_username="user",
        authorized_username_password="password",
    )

    async def open_session():
        print("📡 Opening StreamableHTTP transport connection with auth...")
        async with streamablehttp_client(
            url=server_url,
            auth=oauth_client.oauth,
            timeout=timedelta(seconds=60),
        ) as (read_stream, write_stream, get_session_id):
            print("🤝 Initializing MCP session...")
            async with ClientSession(read_stream, write_stream) as session:
                await session.initialize()
                print("✨ Session initialization complete!")

                print(f"\n✅ Connected to MCP server at {server_url}")
                if get_session_id:
                    session_id = get_session_id()
                    if session_id:
                        print(f"Session ID: {session_id}")

                tools = await session.list_tools()

                print("\n⚙️  Aviable Tools")
                for tool in tools.tools:
                    print(f"   • {tool.name}: {tool.description}")

    asyncio.run(open_session())
